@@ -437,10 +437,7 @@ impl IncomingBatch {
     fn from_acquired(endpoint: &Endpoint, body: AcquireResponseBody) -> Result<IncomingBatch, IncomingError> {
         let url = body.batch_url(endpoint);
 
-        let maybe_pos = VariantPosition::from_setup(body.variant.into(), &body.position, match body.variant {
-            LichessVariant::Chess960 | LichessVariant::FromPosition => CastlingMode::Chess960,
-            _ => CastlingMode::detect(&body.position),
-        });
+        let maybe_pos = VariantPosition::from_setup(body.variant.into(), &body.position, CastlingMode::Chess960);
 
         let (flavor, mut pos) = match maybe_pos {
             Ok(pos @ VariantPosition::Chess(_)) if body.work.is_analysis() => (EngineFlavor::Official, pos),
@@ -448,11 +445,10 @@ impl IncomingBatch {
             Err(pos) => (EngineFlavor::MultiVariant, pos.ignore_impossible_material()?),
         };
 
-        let castling_mode = pos.castles().mode();
         let mut body_moves = Vec::new();
         for uci in body.moves {
             let m = uci.to_move(&pos)?;
-            body_moves.push(m.to_uci(castling_mode));
+            body_moves.push(m.to_uci(CastlingMode::Chess960));
             pos.play_unchecked(&m);
         }
 
@@ -469,7 +465,6 @@ impl IncomingBatch {
                         flavor,
                         position_id: PositionId(0),
                         variant: body.variant,
-                        castling_mode,
                         fen: body.position,
                         moves: body_moves,
                     })]
@@ -485,7 +480,6 @@ impl IncomingBatch {
                         flavor,
                         position_id: PositionId(0),
                         variant: body.variant,
-                        castling_mode,
                         fen: body.position.clone(),
                         moves: moves.clone(),
                     })];
@@ -501,7 +495,6 @@ impl IncomingBatch {
                             flavor,
                             position_id: PositionId(1 + i),
                             variant: body.variant,
-                            castling_mode,
                             fen: body.position.clone(),
                             moves: moves.clone(),
                         }));
